<?php
/**
.---------------------------------------------------------------------.
|  @package: Theme Lazy4basic (a.k.a. Personal)
|  @version: v1.0 (28 June 2018)
|  @link:    http://italinux.com/personal
|  @docs:    http://italinux.com/theme-personal
|
|  @author: Matteo Montanari <matteo@italinux.com>
|  @link:   http://matteo-montanari.com
'---------------------------------------------------------------------'
.---------------------------------------------------------------------------.
| @copyright (c) 2018                                                       |
| ------------------------------------------------------------------------- |
| @license: Concrete5.org Marketplace Commercial Add-Ons & Themes License   |
|           http://concrete5.org/help/legal/commercial_add-on_license       |
|           or just: file://theme_lazy4basic/LICENSE.TXT
|                                                                           |
| This program is distributed in the hope that it will be useful - WITHOUT  |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     |
| FITNESS FOR A PARTICULAR PURPOSE.                                         |
'---------------------------------------------------------------------------'
*/
namespace Concrete\Package\ThemeLazy4basic\Block\L4bWhatIDo;

use Concrete\Package\ThemeLazy4basic\Src\Utils\Utils as BlockUtils;
use Concrete\Core\Block\BlockController;
use Concrete\Core\Asset\Asset;
use Concrete\Core\Asset\AssetList;
use Concrete\Core\Editor\LinkAbstractor;
use Page;
use BlockType;
use Config;

defined('C5_EXECUTE') or die("Access Denied.");

class Controller extends BlockController
{

    protected $btTable = "btLazy4basicWhatIDo";

    protected static $btHandlerId = "what-i-do";
    protected static $btViewPointIdPrefix = "wyp";

    // Total number Items on this Block Grid
    protected static $btItemsTotal = 4;

    protected $btInterfaceWidth = "1355";
    protected $btInterfaceHeight = "550";

    protected $btWrapperClass = 'ccm-ui';
    protected $btWrapperForm = 'lazy-ui mini-wysiwyg';

    protected $btDefaultSet = 'lazy4basic';

    // Custom Image Thumb Width X Height (pixels)
    protected static $btCustomImageThumbWidth = 96;
    protected static $btCustomImageThumbHeight = 96;

    // Style Background & Foreground Colours
    protected static $btStyleOpacity = '0.5';

    // Upload Image size in KBytes (1KB = 1024b)
    protected static $btStyleUploadImageSize = 650;

    // Upload Image Thumb Width X Height (pixels)
    protected static $btStyleUploadThumbWidth = 1680;
    protected static $btStyleUploadThumbHeight = 945;

    // Background Over Image default Opacity
    protected static $bgOverImageOpacity = 1;

    // Support for Inline Editing
    protected $btSupportsInlineEdit = false;
    protected $btSupportsInlineAdd = false;

    // Bootstrap theme Grid Support
    protected $btIgnorePageThemeGridFrameworkContainer = false;

    // Cache block's database calls
    protected $btCacheBlockRecord = true;

    // Cache block's actual view output
    protected $btCacheBlockOutput = false;

    // Serve cached version even if the result of a post request
    protected $btCacheBlockOutputOnPost = true;

    // Server cached version even if user is logged in
    protected $btCacheBlockOutputForRegisteredUsers = true;

    /**
    * When block caching is enabled and output caching is enabled for a block,
    * this is the value in seconds before cache being refreshed. Default (0) is no limit.
    */
    protected $btCacheBlockOutputLifetime = 0;

    protected static function get_btStyles()
    {
        return array(
            'bgColorRGBA' => t('Background Colour'),
            'bgColorOpacity' => t('Adjust Background Opacity'),
            'bgFID' => t('Background Image'),
            'fgColorRGB' => t('Foreground Colour'),
            'isAnimated' => t('Animation / Transition'),
            'o1_isEnabled' => t('1st %s', t('profile')),
            'o1_fID' => t('1st %s', t('custom image')),
            'o2_isEnabled' => t('2nd %s', t('profile')),
            'o2_fID' => t('2nd %s', t('custom image')),
            'o3_isEnabled' => t('3rd %s', t('profile')),
            'o3_fID' => t('3rd %s', t('custom image')),
            'o4_isEnabled' => t('4th %s', t('profile')),
            'o4_fID' => t('4th %s', t('custom image')),
        );
    }

    protected static function get_btFields()
    {
        return array(
            'title' => array(
                'label' => t('Title'),
                'allowEmpty' => true,
            ),
            'subtitle' => array(
                'label' => t('Subtitle'),
                'allowEmpty' => true,
            ),
            'CTA_link' => array(
                'label' => t('CTA %s', t('link')),
                'allowEmpty' => true,
            ),
            'CTA_text' => array(
                'label' => t('CTA %s', t('text')),
                'allowEmpty' => true,
            ),
            'o1_link' => array(
                'label' => t('1st %s', t('link')),
                'allowEmpty' => true,
                'validateLink' => false,
                'validCondition' => 'o1_isEnabled',
            ),
            'o1_icon' => array(
                'label' => t('1st %s', t('icon')),
                'allowEmpty' => true,
            ),
            'o1_title' => array(
                'label' => t('1st %s', t('title')),
                'allowEmpty' => false,
                'validCondition' => 'o1_isEnabled',
            ),
            'o1_content' => array(
                'label' => t('1st %s', t('content')),
                'allowEmpty' => false,
                'validCondition' => 'o1_isEnabled',
            ),
            'o1_button' => array(
                'label' => t('1st %s', t('button text')),
                'allowEmpty' => true,
            ),
            'o2_link' => array(
                'label' => t('2nd %s', t('link')),
                'allowEmpty' => true,
                'validateLink' => false,
                'validCondition' => 'o2_isEnabled',
            ),
            'o2_icon' => array(
                'label' => t('2nd %s', t('icon')),
                'allowEmpty' => true,
            ),
            'o2_title' => array(
                'label' => t('2nd %s', t('title')),
                'allowEmpty' => false,
                'validCondition' => 'o2_isEnabled',
            ),
            'o2_content' => array(
                'label' => t('2nd %s', t('content')),
                'allowEmpty' => false,
                'validCondition' => 'o2_isEnabled',
            ),
            'o2_button' => array(
                'label' => t('2nd %s', t('button text')),
                'allowEmpty' => true,
            ),
            'o3_link' => array(
                'label' => t('3rd %s', t('link')),
                'allowEmpty' => true,
                'validateLink' => false,
                'validCondition' => 'o3_isEnabled',
            ),
            'o3_icon' => array(
                'label' => t('3rd %s', t('icon')),
                'allowEmpty' => true,
            ),
            'o3_title' => array(
                'label' => t('3rd %s', t('title')),
                'allowEmpty' => false,
                'validCondition' => 'o3_isEnabled',
            ),
            'o3_content' => array(
                'label' => t('3rd %s', t('content')),
                'allowEmpty' => false,
                'validCondition' => 'o3_isEnabled',
            ),
            'o3_button' => array(
                'label' => t('3rd %s', t('button text')),
                'allowEmpty' => true,
            ),
            'o4_link' => array(
                'label' => t('4th %s', t('link')),
                'allowEmpty' => true,
                'validateLink' => false,
                'validCondition' => 'o4_isEnabled',
            ),
            'o4_icon' => array(
                'label' => t('4th %s', t('icon')),
                'allowEmpty' => true,
            ),
            'o4_title' => array(
                'label' => t('4th %s', t('title')),
                'allowEmpty' => false,
                'validCondition' => 'o4_isEnabled',
            ),
            'o4_content' => array(
                'label' => t('4th %s', t('content')),
                'allowEmpty' => false,
                'validCondition' => 'o4_isEnabled',
            ),
            'o4_button' => array(
                'label' => t('4th %s', t('button text')),
                'allowEmpty' => true,
            ),
        );
    }

    protected static function get_btWYSIWYG()
    {
        return array(
            'o1_content' => t('1st %s', t('content')),
            'o2_content' => t('2nd %s', t('content')),
            'o3_content' => t('3rd %s', t('content')),
            'o4_content' => t('4th %s', t('content')),
        );
    }

    protected static function get_btFormExtraValues()
    {
        return array(
            'whatIDoIcons' => array(
                'label' => t('Options what I do icons'),
                'allowEmpty' => true,
            ),
            'bgColorOpacityOptions' => array(
                'label' => t('Options adjust background opacity'),
                'allowEmpty' => true,
            ),
            'customTemplateName' => array(
                'label' => t('Custom template name'),
                'allowEmpty' => true,
            ),
        );
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block info Methods
    */
    public function getBlockTypeName()
    {

        return t('L4b What I Do');
    }

    public function getBlockTypeDescription()
    {

        return t('Add L4b What I Do to your website');
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This block Main Methods
    */
    public function getTitle() 
    {
        $cName  = 'title';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = t('what I do');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }
    
    public function getSubtitle() 
    {
        $cName  = 'subtitle';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = ' ';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCTA_link()
    {
        $cName  = 'CTA_link';
        $config = self::$btHandlerId . '.CTA.link';
        $dValue = '#contacts';

        return trim(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}));
    }

    public function getCTA_text()
    {
        $cName  = 'CTA_text';
        $config = self::$btHandlerId . '.CTA.text';
        $dValue = t('ask me more');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO1_isEnabled()
    {
        $cName  = 'o1_isEnabled';
        $config = self::$btHandlerId . '.item.0.isEnabled';
        $dValue = true;

        return filter_var(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}), FILTER_VALIDATE_BOOLEAN);
    }

    public function getO1_fID()
    {
        if ($this->o1_fID > 0) {
            $fObj = BlockUtils::getFileObject($this->o1_fID);
        }

        return (isset($fObj) && is_object($fObj)) ? $fObj : null;
    }

    public function getO1_icon()
    {
        $cName  = 'o1_icon';
        $config = self::$btHandlerId . '.item.0.icon';
        $dValue = 'user-secret';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO1_link()
    {
        $cName  = 'o1_link';
        $config = self::$btHandlerId . '.item.0.link';
        $dValue = 'http://matteo-montanari.com/addons';

        return trim(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}));
    }

    public function getO1_title()
    {
        $cName  = 'o1_title';
        $config = self::$btHandlerId . '.item.0.title';
        $dValue = t('surveillance');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO1_content()
    {
        $cName  = 'o1_content';
        $config = self::$btHandlerId . '.item.0.content';
        $dValue = '<p>Crebritate tunica die et armatis eum et nihil muros <strong>mandato obscuro</strong> Polam prope celeri</p>';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO1_button()
    {
        $cName  = 'o1_button';
        $config = self::$btHandlerId . '.item.0.button';
        $dValue = t('1st %s', t('button text'));

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO2_isEnabled()
    {
        $cName  = 'o2_isEnabled';
        $config = self::$btHandlerId . '.item.1.isEnabled';
        $dValue = true;

        return filter_var(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}), FILTER_VALIDATE_BOOLEAN);
    }

    public function getO2_fID()
    {
        if ($this->o2_fID > 0) {
            $fObj = BlockUtils::getFileObject($this->o2_fID);
        }

        return (isset($fObj) && is_object($fObj)) ? $fObj : null;
    }

    public function getO2_icon()
    {
        $cName  = 'o2_icon';
        $config = self::$btHandlerId . '.item.1.icon';
        $dValue = 'diamond';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO2_link()
    {
        $cName  = 'o2_link';
        $config = self::$btHandlerId . '.item.1.link';
        $dValue = 'http://matteo-montanari.com/themes';

        return trim(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}));
    }

    public function getO2_title()
    {
        $cName  = 'o2_title';
        $config = self::$btHandlerId . '.item.1.title';
        $dValue = t('tracing assets');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO2_content()
    {
        $cName  = 'o2_content';
        $config = self::$btHandlerId . '.item.1.content';
        $dValue = '<p>Miaci celeri non <strong>non pavceids</strong> si hostes rapacium vastabant nec tamen celeri <strong>ultro sinmo</strong> vantur</p>';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO2_button()
    {
        $cName  = 'o2_button';
        $config = self::$btHandlerId . '.item.1.button';
        $dValue = t('2nd %s', t('button text'));

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO3_isEnabled()
    {
        $cName  = 'o3_isEnabled';
        $config = self::$btHandlerId . '.item.2.isEnabled';
        $dValue = true;

        return filter_var(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}), FILTER_VALIDATE_BOOLEAN);
    }

    public function getO3_fID()
    {
        if ($this->o3_fID > 0) {
            $fObj = BlockUtils::getFileObject($this->o3_fID);
        }

        return (isset($fObj) && is_object($fObj)) ? $fObj : null;
    }

    public function getO3_icon()
    {
        $cName  = 'o3_icon';
        $config = self::$btHandlerId . '.item.2.icon';
        $dValue = 'balance-scale';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO3_link()
    {
        $cName  = 'o3_link';
        $config = self::$btHandlerId . '.item.2.link';
        $dValue = '#contacts';

        return trim(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}));
    }

    public function getO3_title()
    {
        $cName  = 'o3_title';
        $config = self::$btHandlerId . '.item.2.title';
        $dValue = t('testimony');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO3_content()
    {
        $cName  = 'o3_content';
        $config = self::$btHandlerId . '.item.2.content';
        $dValue = '<p>Retentabant dubitatur esset Lotophagi domicilium <strong>virtutum humanitatis</strong> ingenuos dubitatur</p>';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO3_button()
    {
        $cName  = 'o3_button';
        $config = self::$btHandlerId . '.item.2.button';
        $dValue = t('check it out');        

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO4_isEnabled()
    {
        $cName  = 'o4_isEnabled';
        $config = self::$btHandlerId . '.item.3.isEnabled';
        $dValue = true;

        return filter_var(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}), FILTER_VALIDATE_BOOLEAN);
    }

    public function getO4_fID()
    {
        if ($this->o4_fID > 0) {
            $fObj = BlockUtils::getFileObject($this->o4_fID);
        }

        return (isset($fObj) && is_object($fObj)) ? $fObj : null;
    }

    public function getO4_icon()
    {
        $cName  = 'o4_icon';
        $config = self::$btHandlerId . '.item.3.icon';
        $dValue = 'institution';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO4_link()
    {
        $cName  = 'o4_link';
        $config = self::$btHandlerId . '.item.3.link';
        $dValue = 'http://matteo-montanari.com/c5';

        return trim(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}));
    }

    public function getO4_title()
    {
        $cName  = 'o4_title';
        $config = self::$btHandlerId . '.item.3.title';
        $dValue = t('process serving');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO4_content()
    {
        $cName  = 'o4_content';
        $config = self::$btHandlerId . '.item.3.content';
        $dValue = '<p>Laeva successorio <strong>Euphratis dextra</strong> cum limes cognomentum regna porrigitur ut efficaciae</p>';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getO4_button()
    {
        $cName  = 'o4_button';
        $config = self::$btHandlerId . '.item.3.button';
        $dValue = t('check it out');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * All icons Available
    */
    public function getWhatIDoIcons()
    {

        return array(
            'address-book',
            'address-book-o',
            'address-card',
            'address-card-o',
            'adjust',
            'american-sign-language-interpreting',
            'anchor',
            'archive',
            'area-chart',
            'assistive-listening-systems',
            'asterisk',
            'at',
            'automobile',
            'audio-description',
            'balance-scale',
            'ban',
            'bank',
            'bar-chart',
            'barcode',
            'bars',
            'battery-3',
            'bed',
            'beer',
            'bell',
            'bell-o',
            'bell-slash',
            'bell-slash-o',
            'bicycle',
            'binoculars',
            'birthday-cake',
            'blind',
            'bolt',
            'bomb',
            'book',
            'bookmark',
            'bookmark-o',
            'braille',
            'briefcase',
            'bug',
            'building',
            'building-o',
            'bullhorn',
            'bullseye',
            'bus',
            'cab',
            'calculator',
            'calendar',
            'calendar-o',
            'calendar-check-o',
            'calendar-minus-o',
            'calendar-plus-o',
            'calendar-times-o',
            'camera',
            'camera-retro',
            'car',
            'caret-square-o-down',
            'caret-square-o-left',
            'caret-square-o-right',
            'caret-square-o-up',
            'cart-plus',
            'cc',
            'certificate',
            'check',
            'check-circle',
            'check-circle-o',
            'check-square',
            'check-square-o',
            'child',
            'circle',
            'circle-o',
            'circle-o-notch',
            'circle-thin',
            'clock-o',
            'clone',
            'close',
            'cloud',
            'cloud-download',
            'cloud-upload',
            'code',
            'code-fork',
            'coffee',
            'cog',
            'cogs',
            'comment',
            'comment-o',
            'comments',
            'comments-o',
            'commenting',
            'commenting-o',
            'compass',
            'copyright',
            'credit-card',
            'credit-card-alt',
            'creative-commons',
            'crop',
            'crosshairs',
            'cube',
            'cubes',
            'cutlery',
            'dashboard',
            'database',
            'deaf',
            'desktop',
            'diamond',
            'dot-circle-o',
            'download',
            'drivers-license',
            'drivers-license-o',
            'edit',
            'ellipsis-h',
            'ellipsis-v',
            'envelope',
            'envelope-o',
            'envelope-open',
            'envelope-open-o',
            'envelope-square',
            'eraser',
            'exchange',
            'exclamation',
            'exclamation-circle',
            'exclamation-triangle',
            'external-link',
            'external-link-square',
            'eye',
            'eye-slash',
            'eyedropper',
            'fax',
            'female',
            'fighter-jet',
            'file-archive-o',
            'file-audio-o',
            'file-code-o',
            'file-excel-o',
            'file-image-o',
            'file-movie-o',
            'file-pdf-o',
            'file-photo-o',
            'file-picture-o',
            'file-powerpoint-o',
            'file-sound-o',
            'file-video-o',
            'file-word-o',
            'file-zip-o',
            'film',
            'filter',
            'fire',
            'fire-extinguisher',
            'flag',
            'flag-checkered',
            'flag-o',
            'flash',
            'flask',
            'folder',
            'folder-o',
            'folder-open',
            'folder-open-o',
            'frown-o',
            'futbol-o',
            'gamepad',
            'gavel',
            'gear',
            'gears',
            'genderless',
            'gift',
            'glass',
            'globe',
            'graduation-cap',
            'group',
            'hard-of-hearing',
            'hdd-o',
            'handshake-o',
            'hashtag',
            'headphones',
            'heart',
            'heart-o',
            'heartbeat',
            'history',
            'home',
            'hotel',
            'hourglass',
            'hourglass-1',
            'hourglass-2',
            'hourglass-o',
            'i-cursor',
            'id-badge',
            'id-card',
            'id-card-o',
            'image',
            'inbox',
            'industry',
            'info',
            'info-circle',
            'institution',
            'key',
            'keyboard-o',
            'language',
            'laptop',
            'leaf',
            'legal',
            'lemon-o',
            'level-down',
            'level-up',
            'life-saver',
            'lightbulb-o',
            'line-chart',
            'location-arrow',
            'lock',
            'low-vision',
            'magic',
            'magnet',
            'mail-forward',
            'mail-reply',
            'mail-reply-all',
            'male',
            'map',
            'map-o',
            'map-pin',
            'map-signs',
            'map-marker',
            'meh-o',
            'microchip',
            'microphone',
            'microphone-slash',
            'minus',
            'minus-circle',
            'minus-square',
            'minus-square-o',
            'mobile',
            'money',
            'moon-o',
            'mortar-board',
            'motorcycle',
            'mouse-pointer',
            'music',
            'navicon',
            'newspaper-o',
            'object-group',
            'object-ungroup',
            'paint-brush',
            'paper-plane',
            'paper-plane-o',
            'paw',
            'pencil',
            'pencil-square',
            'pencil-square-o',
            'percent',
            'phone',
            'phone-square',
            'picture-o',
            'pie-chart',
            'plane',
            'plug',
            'plus',
            'plus-circle',
            'plus-square',
            'plus-square-o',
            'podcast',
            'power-off',
            'print',
            'puzzle-piece',
            'qrcode',
            'question',
            'question-circle',
            'question-circle-o',
            'quote-left',
            'quote-right',
            'random',
            'recycle',
            'refresh',
            'registered',
            'remove',
            'reorder',
            'retweet',
            'road',
            'rocket',
            'rss',
            'rss-square',
            's15',
            'search',
            'search-minus',
            'search-plus',
            'send',
            'send-o',
            'server',
            'share-alt',
            'share-alt-square',
            'share-square',
            'share-square-o',
            'shield',
            'ship',
            'shopping-bag',
            'shopping-basket',
            'shopping-cart',
            'shower',
            'sign-in',
            'sign-out',
            'sign-language',
            'signal',
            'sitemap',
            'sliders',
            'smile-o',
            'snowflake-o',
            'soccer-ball-o',
            'sort',
            'sort-alpha-asc',
            'sort-alpha-desc',
            'sort-amount-asc',
            'sort-amount-desc',
            'sort-asc',
            'sort-desc',
            'sort-numeric-asc',
            'sort-numeric-desc',
            'space-shuttle',
            'spinner',
            'spoon',
            'square',
            'square-o',
            'star',
            'star-half',
            'star-half-empty',
            'star-o',
            'sticky-note',
            'sticky-note-o',
            'street-view',
            'suitcase',
            'sun-o',
            'support',
            'tablet',
            'tachometer',
            'tag',
            'tags',
            'tasks',
            'taxi',
            'television',
            'terminal',
            'thermometer-0',
            'thermometer-2',
            'thermometer-4',
            'thumb-tack',
            'thumbs-down',
            'thumbs-o-up',
            'thumbs-up',
            'ticket',
            'times',
            'times-circle',
            'times-circle-o',
            'times-rectangle',
            'times-rectangle-o',
            'tint',
            'toggle-off',
            'toggle-on',
            'trademark',
            'trash',
            'trash-o',
            'tree',
            'trophy',
            'truck',
            'tty',
            'tv',
            'umbrella',
            'universal-access',
            'university',
            'unlock',
            'unlock-alt',
            'unsorted',
            'upload',
            'user',
            'user-circle',
            'user-circle-o',
            'user-o',
            'user-plus',
            'user-secret',
            'user-times',
            'users',
            'video-camera',
            'volume-control-phone',
            'volume-down',
            'volume-off',
            'volume-up',
            'warning',
            'wheelchair',
            'wheelchair-alt',
            'window-close',
            'window-close-o',
            'window-maximize',
            'window-restore',
            'wifi',
            'wrench',
            'facebook'
        );
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Get All enabled Profiles
    */
    public function getAll_isEnabled($range)
    {
        $o = array();

        foreach ($range as $value) {
            $o[$value] = $this->{'getO'.$value.'_isEnabled'}();
        }

        return array_diff($o, array(false));
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Get All data Profiles to display
    */
    public function getAll_data($profiles)
    {

        $o = array();
        $offset = 0;

        foreach (array_keys($this->getAll_isEnabled($profiles)) as $key) {

        $offset++;

            $o[$key] = array(
                'col-lg' => $this->getBootstrapCol_LG_Config($offset),
                'col-md' => $this->getBootstrapCol_MD_Config($offset),
                'col-sm' => $this->getBootstrapCol_SM_Config($offset),
                'col-sx' => $this->getBootstrapCol_SX_Config($offset),
                 'class' => BlockUtils::getClassByLink($this->{'getO'.$key.'_link'}()),
                  'link' => $this->{'getO'.$key.'_link'}(),
                'target' => BlockUtils::getTargetByLink($this->{'getO'.$key.'_link'}()),
                   'img' => array(
                      'src' => $this->getBlockForegroundImageURL($this->{'getO'.$key.'_fID'}()),
                   ),
                  'icon' => array(
                      'tag' => $this->{'getO'.$key.'_icon'}(),
                  ),
                 'title' => $this->{'getO'.$key.'_title'}(),
               'content' => $this->{'getO'.$key.'_content'}(),
                'button' => $this->{'getO'.$key.'_button'}(),
            );
        }

        return $o;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This block GRID Methods
    */
    public function getBootstrapCol_LG_Config($offset)
    {
        return $this->getBootstrapCol_Config($offset, 'lg');
    }

    public function getBootstrapCol_MD_Config($offset)
    {
        return $this->getBootstrapCol_Config($offset, 'md');
    }

    public function getBootstrapCol_SM_Config($offset)
    {
        return $this->getBootstrapCol_Config($offset, 'sm');
    }

    public function getBootstrapCol_SX_Config($offset)
    {
        return $this->getBootstrapCol_Config($offset, 'sx');
    }

    public function getBootstrapCol_Config($offset, $key)
    {

        $array = $this->getAll_isEnabled(range(1, self::$btItemsTotal));

        // Total number of items
        $items = count($array);

        // lg, md, sm, sx
        $lgMax = $mdMax = $smMax = $sxMax = null;

        // Offsets
        $lgOffset = $mdOffset = $smOffset = $sxOffset = 0;

        /* Choose Max number of items per view type:
         *
         * LG = Large Screen
         * MD = Desktop
         * SM = Tablet
         * SX = Mobile
         */
        // case the number of items to display and so how
        switch($items) {
        case 4:
           $lgMax = 4; $mdMax = 3; $smMax = 2; $sxMax = 1;

           // add Offset manually here if required
           if ($offset == 4) {
               $mdOffset = 1;
           }
           break;
        case 3:
           $lgMax = 3; $mdMax = 3; $smMax = 2; $sxMax = 1;

           // add Offset manually here if required
           if ($offset == 3) {
               $smOffset = 1;
           }
           break;
        case 2:
           $lgMax = 2; $mdMax = 2; $smMax = 2; $sxMax = 1;
           break;
        case 1:
           $lgMax = 1; $mdMax = 1; $smMax = 1; $sxMax = 1;
           break;
        }

        // Calculate the number of columns requited in the View Layout per number of items chosen
        ${$key . 'Max'} = is_int(${$key . 'Max'}) ? (12 / ${$key . 'Max'}) : null;

        // Calculate Offset if required
        ${$key . 'Offset'} = ${$key . 'Offset'} == true ? ($items * ${$key . 'Offset'}) : 0;

        // set the value into an array
        $data = ${$key . 'Max'} == true ? array(${$key . 'Max'}, 'offset' => ${$key . 'Offset'}) : array();


        return $data;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This Block RegisterViewAssets
    */
    public function registerViewAssets($outputContent = '')
    {
        // Import JS core Theme
        $this->requireAsset('javascript', 'jquery');

        if ($this->getIsAnimated() === true && Page::getCurrentPage()->isEditMode() == false) {
            // Import Animations CSS & JS Configuration
            $this->requireAsset('jst.animate.conf');
        }
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This Block View/Validate/Save Methods
    */
    public function view()
    {

        // Set style values
        $this->set('sID', $this->getSectionId());
        $this->set('viewPoint', self::getViewPointId());
        $this->set('cTemplate', $this->getCustomTemplateName());
        $this->set('cFgColorClass', $this->getCustomFgColorClassName());

        // Sanitize WYSIWYGs inputs
        foreach (array_keys(self::get_btWYSIWYG()) as $key) {
            $this->set($key, LinkAbstractor::translateFrom($this->{'get' . ucfirst($key)}()));
        }

        // Sanitize & set some of the main values
        $this->set('allData', $this->getAll_data(range(1, self::$btItemsTotal)));

        // global CTA button
        $this->set('CTA', array('text' => $this->getCTA_text(),
                               'class' => BlockUtils::getClassByLink($this->getCTA_link()),
                                'link' => $this->getCTA_link(),
                              'target' => BlockUtils::getTargetByLink($this->getCTA_link())));

        // Import Custom Css3 inline
        $this->set('cStyle', $this->getCustomStyle());
    }

    public function on_start()
    {
        $al = AssetList::getInstance();

        $cf = Array(
            'position' => Asset::ASSET_POSITION_FOOTER,
            'minify' => false,
            'combine' => false
        );

        // Register Assets this Block
        // Register Assets Animate Configuration
        $al->register('javascript', $this->getJSelectorId() . '.animate-conf', 'blocks/l4b_what_i_do/jscript/lazy-animate.conf.js', $cf, 'theme_lazy4basic');
        $al->register('javascript-inline', $this->getJSelectorId() . '.animate-init',  '$("section#' . $this->getSectionId()  . '").lazyAnimateInit();', $cf, 'theme_lazy4basic');

        $al->registerGroup(
            'jst.animate.conf', array(
               array(
                   'javascript',
                   $this->getJSelectorId() . '.animate-conf'
               ),
               array(
                   'javascript-inline',
                   $this->getJSelectorId() . '.animate-init'
               ),
            )
        );
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom inline Style main
    * Stream
    */
    protected function getCustomStyle()
    {
        $o=null;

        if (BlockUtils::isValidColor($this->bgColorRGBA) ||
            BlockUtils::isValidImage($this->getBgFID()) ||
            BlockUtils::isValidColor($this->fgColorRGB) ||
            $this->isCustomOverImageOpacity($this->bgColorOpacity)) {

            ob_start();
            ?>

            <style>
              <?php
                if (BlockUtils::isValidColor($this->bgColorRGBA) ||
                    $this->isCustomOverImageOpacity($this->bgColorOpacity)) {
                ?>
                  section<?php echo $this->getStyleSelector()?>.over-image::before {
                    <?php
                      if (BlockUtils::isValidColor($this->bgColorRGBA)) { ?>
                          background-color: <?php echo $this->getOverImageBgColor()?> !important;
                    <?php } ?>
                    <?php
                      if ($this->isCustomOverImageOpacity($this->bgColorOpacity)) { ?>
                          opacity: <?php echo $this->bgColorOpacity?> !important;
                    <?php } ?>
                  }
              <?php } ?>

              <?php
                if (BlockUtils::isValidImage($this->getBgFID()) ||
                    BlockUtils::isValidColor($this->bgColorRGBA) ||
                    BlockUtils::isValidColor($this->fgColorRGB)) {
                ?>
                  section<?php echo $this->getStyleSelector()?>.over-image {
                    <?php
                      if (BlockUtils::isValidImage($this->getBgFID())) { ?>
                          background-image: url('<?php echo $this->getCustomStyleImagePath()?>') !important;
                    <?php } ?>

                    <?php
                      if (BlockUtils::isValidColor($this->bgColorRGBA)) { ?>
                          background-color: <?php echo $this->bgColorRGBA?> !important;
                    <?php } ?>

                    <?php
                      if (BlockUtils::isValidColor($this->fgColorRGB)) { ?>
                          color: <?php echo $this->fgColorRGB ?> !important;
                    <?php } ?>
                  }
               <?php } ?>

               <?php
                 if (BlockUtils::isValidColor($this->bgColorRGBA)) { ?>
                   section<?php echo $this->getStyleSelector()?> p {
                       text-shadow: 0px 0px 2px <?php echo BlockUtils::getCustomStyleTextShadow($this->bgColorRGBA)?> !important;
                       }
               <?php } ?>
               <?php
                 if (BlockUtils::isValidColor($this->fgColorRGB)) { ?>
                   section<?php echo $this->getStyleSelector()?> .fa {
                       text-shadow: 0px 0px 3px <?php echo BlockUtils::getCustomStyleTextShadow($this->fgColorRGB)?> !important;
                       }
               <?php } ?>
            </style>

            <?php
            $o = ob_get_contents();

            ob_end_clean();

            $o = BlockUtils::getCustomStyleSanitised($o);
        }

        return $o;
    }
 
    public function validate($args)
    {
        $e = BlockUtils::getThisApp()->make('error');

        // Validate Styles
        foreach (self::get_btStyles() as $key => $value) {

            switch ($key) {
            case (lcfirst(substr($key, -3)) == 'fID'):

                if (!empty($args[$key])) {
                    $f = BlockUtils::getFileObject($args[$key]);

                    if (is_object($f)) {
                        switch ($f->getMimeType()) {
                        case 'image/jpeg':
                        case 'image/jpg':
                        case 'image/png':
                            break;
                        default:
                            $e->add(t('File type required: JPG or PNG'));
                            break;
                        }

                        switch (strtolower($f->getExtension())) {
                        case 'jpeg':
                        case 'jpg':
                        case 'png':
                            break;
                        default:
                            $e->add(t('File extension required: JPG or PNG'));
                            break;
                        }

                        $uploadImageSize = BlockUtils::getUploadImageSize(self::$btStyleUploadImageSize);

                        if ($f->getFullSize() > $uploadImageSize) {
                            $e->add(t('File size exceeded: Max %s', BlockUtils::getHumanReadUploadImageSize($uploadImageSize)));
                        }

                        if ($f->isError()) {
                            $e->add(t('File image is invalid'));
                        }
                    } else {
                        $e->add(t('File image is invalid'));
                    }
                }
                break;
            }
        }

        // Validate Fields
        foreach (self::get_btFields() as $key => $value) {

            // Field is Empty
            if (isset($value['allowEmpty']) && ($value['allowEmpty'] === false)) {

                if (isset($value['validCondition'])) {
                    if (empty($args[$key]) && (empty($args[$value['validCondition']]) == false)) {
                        $e->add(t('Cannot be empty: %s', $value['label']));
                    }
                } else {
                    if (empty($args[$key])) {
                        $e->add(t('Cannot be empty: %s', $value['label']));
                    }
                }
            }

            // Field is Link
            if (isset($value['validateLink']) && ($value['validateLink'] === true) && isset($value['validCondition'])) {
                if ((BlockUtils::getIsValidLink($args[$key]) === false) && (empty($args[$value['validCondition']]) == false)) {
                    $e->add(t('Invalid link: %s', $value['label']));
                }
            }
        }

        return $e;
    }
 
    public function save($args)
    {

        // Update custom Styles for all pages 
        foreach (self::get_btStyles() as $key => $value) {

            switch ($key) {
            case (lcfirst(substr($key, -3)) == 'fID'):
                if (empty($args[$key])) {
                    $args[$key] = 0;
                }
                break;
            case 'bgColorRGBA':
                if (empty($args[$key])) {
                    $args[$key] = 'transparent';
                }
                break;
            case 'fgColorRGB':
                if (empty($args[$key])) {
                    $args[$key] = 'transparent';
                }
                break;
            }
        }

        // Sanitize input for DB
        foreach (self::get_btFields() as $key => $value) {

            // Prevent Default Value from setting
            if (empty($args[$key])) {
                $args[$key] = ' ';
            }
        }

        // WYSIWYGs inputs
        foreach (self::get_btWYSIWYG() as $key => $value) {

            // Strip 'End Of Line' symbol for this platform ("\n" or "\r")
            $args[$key] = trim(str_replace(PHP_EOL, '', $args[$key]));

            // Sanitize URLs
            $args[$key] = LinkAbstractor::translateTo($args[$key]);
        }

        parent::save($args);
    }
 
    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block Handlers Design Methods
    */
    protected static function getViewPointId()
    {
        return self::$btViewPointIdPrefix . '-' . self::$btHandlerId;
    }

    protected function getSectionId()
    {
        return 's' . $this->bID;
    }

    protected function getStyleSelector()
    {
        return '#' . $this->getSectionId() . '.' . self::$btHandlerId;
    }

    protected function getJSelectorId()
    {
        return $this->getSectionId() . '.' . self::$btHandlerId;
    }

    protected function getCustomTemplateName()
    {
        $tName = 'no_template';
        $block = $this->getBlockObject();

        if (is_object($block)) {
          $tName = ($block->getBlockFilename() == true ? $block->getBlockFilename() : $tName);
        } 

        return $tName;
    }

    protected function getCustomFgColorClassName()
    {
        return (BlockUtils::isValidColor($this->fgColorRGB) ? 'cfg-color' : null);
    }

    public static function getBlockHandle()
    {
        return strtolower(basename(dirname(__FILE__)));
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block - Foreground Custom Images PATHs
    *
    * @return image full Path (file system)
    */
    protected function getBlockForegroundImageURL($fID, $w = null, $h = null)
    {
        // set thumbnail width
        $width = ($w == false) ? self::$btCustomImageThumbWidth : $w;

        // set thumbnail height
        $height = ($h == false) ? self::$btCustomImageThumbHeight : $h;

        // get source thumbnail image
        return (is_object($fID) ? BlockUtils::getThisApp()->make('helper/image')->getThumbnail($fID, $width, $height, true)->src : null);
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Default & Background Images URLs
    *
    * @return image URL (for HTML src attribute)
    */
    protected function getBlockDefaultImageURL()
    {
        $image = $this->getDefaultImageFullSrc();

        return $image['url'];
    }

    protected function getBlockBackgroundImageURL()
    {
        $image = $this->getBackgroundImageFullSrc();

        return $image['url'];
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Default & Background Images PATHs
    *
    * @return image full Path (file system)
    */
    protected function getBlockDefaultImagePath()
    {
        $image = $this->getDefaultImageFullSrc();

        return $image['path'];
    }

    protected function getBlockBackgroundImagePath()
    {
        $image = $this->getBackgroundImageFullSrc();

        return $image['path'];
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Default Images Width & Height
    *
    * @return image full Path (file system)
    */
    protected function getBlockDefaultImageWidth()
    {
        $size = @getimagesize($this->getBlockDefaultImagePath());

        // get default image width
        return $size[0];
    }

    protected function getBlockDefaultImageHeight()
    {
        $size = @getimagesize($this->getBlockDefaultImagePath());

        // get default image height
        return $size[1];
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Paths & URLs 
    *
    * @param true / false
    * @return image base Path and URL (Template / Block)
    */
    protected function getBlockImagesBaseURL($tPath=false)
    {
        return $this->getBlockAssetsURL() . ($tPath !== false ? $this->getBlockTemplateRelativePath() : null) . '/images';
    }

    protected function getBlockImagesBasePath($tPath=false)
    {
        return __DIR__ . ($tPath !== false ? $this->getBlockTemplateRelativePath() : null) . '/images';
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template - Paths & URLs 
    *
    * @return image base Path and URL for a Template
    */
    protected function getBlockImagesBaseTemplateURL()
    {
        return $this->getBlockImagesBaseURL(true);
    }

    protected function getBlockImagesBaseTemplatePath()
    {
        return $this->getBlockImagesBasePath(true);
    }

    /** Get Template relative Path */
    protected function getBlockTemplateRelativePath()
    {
        return ($this->getCustomTemplateName() == false ? null : '/templates/' . $this->getCustomTemplateName());
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block functional Methods
    */
    private function getBackgroundImageFullSrc()
    {
        return $this->getDefaultImageFullSrc('background');
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * get Template or Block images Paths & URLs 
    * IMPORTANT: it uses inheritance
    * Returns image from Template or Block folders
    *
    * @param default / background
    * @return array (path, url)
    */
    private function getDefaultImageFullSrc($file='default')
    {
        $image = null;
        $ext = array('png', 'jpg');

        // check if Template Image folder has images 
        if (is_dir($this->getBlockImagesBaseTemplatePath())) {
            foreach ($ext as $value) {
                $imgPath = $this->getBlockImagesBaseTemplatePath() . '/' . $file . '.' . $value;
                
                if (is_file($imgPath)) {
                    $image = array('path' => $imgPath,
                                   'url' => $this->getBlockImagesBaseTemplateURL() . '/' . $file . '.' . $value);
                } 
            }
        } 

        // otherwise use images within Block 
        if ($image == false) {
            foreach ($ext as $value) {
                $imgPath = $this->getBlockImagesBasePath() . '/' . $file . '.' . $value;
                
                if (is_file($imgPath)) {
                    $image = array('path' => $imgPath,
                                   'url' => $this->getBlockImagesBaseURL() . '/' . $file . '.' . $value);
                } 
            }
        }

        return (array) $image;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom inline Style Methods
    * Background inline Styles Methods
    */
    public function getBgFID()
    {
        if ($this->bgFID > 0) {
            $fObj = BlockUtils::getFileObject($this->bgFID);
        }

        return (isset($fObj) && is_object($fObj)) ? $fObj : null;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Check if Animation is enabled (ideal for JS variables)
    */
    public function getIsAnimationEnabled()
    {        
        return ($this->getIsAnimated() === true && Page::getCurrentPage()->isEditMode() == false) ? 'true' : 'false';
    }

    public function getIsEditMode()
    {
        return (Page::getCurrentPage()->isEditMode() == false) ? 0 : 1;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom Animation / Transition
    */
    public function getIsAnimated()
    {
        $cName  = 'isAnimated';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = true;

        return filter_var(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}), FILTER_VALIDATE_BOOLEAN);
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Background Opacity Over Image
    */
    public function getBgColorOpacity()
    {
        $cName  = 'bgColorOpacity';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = self::$bgOverImageOpacity;

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Background Opacity Over Image Options
    */
    public function getBgColorOpacityOptions()
    {
        return array(
            '-75%' => 0.25,
            '-50%' => 0.5,
            '-25%' => 0.75,
            'default' => 1
        );
    }

    protected function getCustomStyleImagePath()
    {
        return BlockUtils::getThisApp()->make('helper/image')->getThumbnail($this->getBgFID(), self::$btStyleUploadThumbWidth, self::$btStyleUploadThumbHeight, false)->src;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom inline Style Methods
    */
    protected function isCustomOverImageOpacity($value)
    {
        return ($value == true && (self::$bgOverImageOpacity != $this->getBgColorOpacity())) === true ? true : false;
    }

    protected function getOverImageBgColor()
    {
        return $this->bgColorRGBA;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Form Overlay (Edit/Add) Methods
    */
    public function getBlockAssetsURL()
    {
        $bt = BlockType::getByHandle(self::getBlockHandle());
        $bPath = BlockUtils::getThisApp()->make('helper/concrete/urls')->getBlockTypeAssetsURL($bt);

        return $bPath;
    }

    protected function addLocalAssets($path, $type = 'css')
    {
        $this->addHeaderItem(BlockUtils::getThisApp()->make('helper/html')->{$type}($this->getBlockAssetsURL() . $path));
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Add Form (Window Overlay) - default Values
    */
    protected function addFormDefaultValues()
    {
        // Retrieve defaults Values
        if (method_exists(__CLASS__, 'get_btFields')) {

            foreach (self::get_btFields() as $key => $value) {

                if (method_exists($this, 'get' . ucfirst($key))) {

                    $o = $this->{'get' . ucfirst($key)}();

                    $o = is_array($o) ? $o : trim($o);

                    $this->set($key, $o);
                }
            }
        }

        // Retrieve WYSIWYG Editor Values translation
        if (method_exists(__CLASS__, 'get_btWYSIWYG')) {
            foreach (self::get_btWYSIWYG() as $key => $value) {
                $this->set($key, LinkAbstractor::translateFrom($this->{'get' . ucfirst($key)}()));
            }
        }
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Add Form (Window Overlay) - extra Values
    */
    protected function addFormExtraValues()
    {
        // Retrieve defaults Values
        if (method_exists(__CLASS__, 'get_btFields')) {
            foreach (array_keys(self::get_btStyles()) as $key) {
                if (method_exists($this, 'get' . ucfirst($key))) {
                    $this->set($key, $this->{'get' . ucfirst($key)}());
                }
            }
        }

        // Retrieve extra Values
        if (method_exists(__CLASS__, 'get_btFormExtraValues')) {
            foreach (array_keys(self::get_btFormExtraValues()) as $key) {
                if (method_exists($this, 'get' . ucfirst($key))) {
                    $this->set($key, $this->{'get' . ucfirst($key)}());
                }
            }
        }
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Add Form (Window Overlay)
    */
    protected function add()
    {
        // WYSIWYG Editor
        if (is_array(self::get_btWYSIWYG())) {

            $editor = BlockUtils::getThisApp()->make('editor');

            $editor->setAllowFileManager(true);
            $editor->setAllowSitemap(false);

            // We can include and exclude editor plugins.
            $editor->getPluginManager()->deselect(array('table', 'specialcharacters'));
            $editor->getPluginManager()->select('fontsize');

            $this->set('editor', $editor);
        }

        // Custom Styles Palettes
        $this->set('color', BlockUtils::getThisApp()->make('helper/form/color'));
        $this->set('asset', BlockUtils::getThisApp()->make('helper/concrete/asset_library'));

        $this->set('fgColorPalette', BlockUtils::getFgColorPalette(false, true));
        $this->set('bgColorPalette', BlockUtils::getBgColorPalette(true, true, self::$btStyleOpacity));

        $this->set('btWrapperForm', $this->btWrapperForm);

        // User Interface
        $this->set('hUI', BlockUtils::getThisApp()->make('helper/concrete/ui'));

        $this->addFormDefaultValues();
        $this->addFormExtraValues();

        // Add Assets to Window Overlay
        $this->addLocalAssets('/css/tools/lazy-global-ui.css', 'css');
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Edit Form (Window Overlay)
    */
    protected function edit() 
    {
        $this->add();
    }
}
