/**
* ============================================
* File type: Theme Extra
* File provides: Smooth Scrolling on querying direct UrLs with anchor
*           es1: http://localhost/#contacts (Single Page)
*           es2: http://localhost/page/#contacts (Multi Pages)
*
* Suggested: .haccess for cleaner URLs
*
* @author: Matteo Montanari <matteo@italinux.com>
*
* TERMS OF USE
* Open source under the MIT License.
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
* IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* ============================================
*/

$(function() {

    /**
    * @preserve
    * INIT Scrolling on URL
    */
    if ($(location).attr("hash")) {

        initScrollURL();
    }

    /**
    * @preserve
    * ============================================
    * EXTERNAL:
    *   Scrolling through Direct Url INTERNET
    *   Main sections listed
    *   with speed in milliseconds
    * ============================================
    */
    function initScrollURL() {

        var delayMsecs = 1e3; // (10 x 10 x 10) = 1000 = 1sec
        var speedFactor = 2.5; // + o - speed (play with decimals first)

        /**
        * @preserve
        * List of Handlers for Scrolling Direct
        * Add or Edit with new #wyp / new add-ons
        *
        * Customise with + offset
        * Add-ons have same config in blocks/jscript (internal scrolling)
        * Change Both!
        */
        var options = {
            'menu': { offset: 0 },
            'banner': { offset: 0 },
            'what-i-do': { offset: 0 },
            'content-free': { offset: 0 },
            'team': { offset: 0 },
            'services': { offset: 0 },
            // 'fb-page-posts': { offset: 0 },
            'about-me': { offset: 0 },
            // 'content-free': { offset: 0 },
            // 'adverts': { offset: 0 },
            // 'bookmarks': { offset: 0 },
            'my-skills': { offset: 5 },
            'curriculum-vitae': { offset: 0 },
                'curriculum-vitae-skills': { offset: 90 },
                'curriculum-vitae-resume': { offset: 90 },
            'portfolio': { offset: 0 },
            'clients': { offset: 0 },
            // 'my-works': { offset: 0 },
            'prices': { offset: 0 },
            // 'testimonials': { offset: 0 },
            'social-media': { offset: 0 },
            'contacts': { offset: 5 },
                'contacts-more': { offset: 90 },
                    'contacts-more-mobile': { offset: 90 },
                    'contacts-more-address': { offset: 90 },
                    'contacts-more-email': { offset: 90 },
            // 'map': { offset: 0 },
            'footer': { offset: 0 },
        };

        /** ============================================
        *  default values do NOT touch BELOW this line.
        */
        var animDurationMsecsDefault = 1e3;

        /**
        * The offset assigned by default if set to 0 (zero)
        */
        var offsetDefault = 50;

        /**
        * Add a base offset in case of elements changing positions while scrolling
        * (e.g. Sticky Menu)
        */
        var baseOffset = 0;

        /**
        * Check devices resolutions where that's the case
        */
        var isTablet = window.matchMedia("only screen and (max-width: 1024px)");

        /**
        * Then change base Offset to match the dynamic element height
        */
        if ((document.querySelector('section.sticky.menu') !== null) && (isTablet.matches === false)) {
            baseOffset = 70;
        }

        var thisConfig;
        var thisHandler;

        var thisHash = $(location).attr("hash").substring(1);
        var parentHash = thisHash.substr(0, thisHash.lastIndexOf("-"));

        var offset;

        var timer;

        // Loop through main Object
        if (thisHash in options) {
            thisConfig = options[thisHash];
        } else if (parentHash in options) {

            var thisParent = options[parentHash];
            var thisChild  = ((typeof thisParent !== 'undefined') && (typeof thisParent[thisHash] !== 'undefined')) ? thisParent[thisHash] : false;

                // create current config obj and its fallback
                thisConfig = (thisChild) ? thisChild : thisParent;
        }

        // set Object Values
        if (thisConfig) {

            // get current handler or its fallback
            thisHandler = "#wyp-" + ($("#wyp-" + thisHash).length ? thisHash : parentHash);


            // detect handler if Any
            if ($(thisHandler).length) {

                // get handler Offset
                var thisHandlerOffset = $(thisHandler).first().offset().top;

                // calculate speed (animation)
                var animDurationMsecs = Math.ceil(thisHandlerOffset / speedFactor);

                var thisAnimDurationMsecs = (animDurationMsecs) ? animDurationMsecs : animDurationMsecsDefault;
                            var postOffset = baseOffset + ((thisConfig.offset) ? thisConfig.offset : offsetDefault);

                // Scroll Animate
                clearTimeout(timer);

                timer = setTimeout(function() {
                    $("html, body").animate({
                        scrollTop: thisHandlerOffset - postOffset
                    }, thisAnimDurationMsecs, "swing");
                }, delayMsecs);
            }
        }
    }
});
